import { Type } from 'extract-pg-schema';
import { ConnectionConfig } from 'pg';
import Matcher from './Matcher';
import { Model } from './Model';
declare type BuiltinType = string;
declare type ImportedType = {
    name: string;
    module: string;
    absoluteImport: boolean;
    defaultImport: boolean;
};
export declare type TypeDefinition = BuiltinType | ImportedType;
export declare type TypeMap = {
    [index: string]: TypeDefinition;
};
export declare type Hook<T> = (lines: string[], src?: T) => string[];
export declare type GivenName = string & {
    __brand?: 'given-name';
};
export declare const nameIdentity: (name: string) => GivenName;
export declare type Nominators = {
    modelNominator?: (modelName: string) => GivenName;
    propertyNominator?: (propertyName: string, model: Model) => string;
    initializerNominator?: (givenName: GivenName, modelName: string) => string;
    idNominator?: (givenName: GivenName, modelName: string) => string;
    typeNominator?: (typeName: string) => GivenName;
    fileNominator?: (givenName: GivenName, originalName: string) => string;
};
declare type Settings = {
    modelFolder?: string;
    ignore?: Matcher[];
    preDeleteModelFolder?: boolean;
    customTypeMap?: TypeMap;
    makeIdType?: (innerType: string, modelName: string) => string;
    modelHooks?: Hook<Model>[];
    typeHooks?: Hook<Type>[];
    resolveViews?: boolean;
} & Nominators;
export declare type SchemaConfig = {
    name: string;
    externalTypesFolder?: string;
} & Settings;
declare type Config = {
    connection?: ConnectionConfig;
    schemas: SchemaConfig[];
} & Settings;
export default Config;
