"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = __importDefault(require("path"));
var ramda_1 = require("ramda");
var generateInterface_1 = __importDefault(require("./generateInterface"));
var importGenerator_1 = __importDefault(require("./importGenerator"));
var generateCompositeTypeFile = function (compositeType, _a) {
    var typeMap = _a.typeMap, userTypes = _a.userTypes, tableOrViewTypes = _a.tableOrViewTypes, schemaName = _a.schemaName, nominators = _a.nominators, externalTypesFolder = _a.externalTypesFolder, schemaFolderMap = _a.schemaFolderMap;
    var fileNominator = nominators.fileNominator;
    var lines = [];
    var comment = compositeType.comment;
    var importGenerator = new importGenerator_1.default(schemaFolderMap[schemaName]);
    var appliedUserTypes = (0, ramda_1.uniq)((0, ramda_1.map)(function (p) { return p.type; }, (0, ramda_1.filter)(function (p) { return userTypes.indexOf(p.type) !== -1; }, compositeType.attributes)));
    appliedUserTypes.forEach(function (t) {
        var givenName = nominators.typeNominator(t);
        importGenerator.addImport(givenName, true, path_1.default.join(schemaFolderMap[schemaName], fileNominator(givenName, t)), false);
    });
    // Handle tables and views used as attribute types.
    var appliedTableOrViewTypes = (0, ramda_1.uniq)((0, ramda_1.map)(function (p) { return p.type; }, (0, ramda_1.filter)(function (p) { return tableOrViewTypes.indexOf(p.type) !== -1; }, compositeType.attributes)));
    appliedTableOrViewTypes.forEach(function (t) {
        var givenName = nominators.modelNominator(t);
        importGenerator.addImport(givenName, true, path_1.default.join(schemaFolderMap[schemaName], fileNominator(givenName, t)), false);
    });
    var overriddenTypes = (0, ramda_1.map)(function (p) { return p.tags.type; }, (0, ramda_1.filter)(function (p) { return Boolean(p.tags.type); }, compositeType.attributes));
    (0, ramda_1.forEach)(function (importedType) {
        var givenName = importedType; // We expect people to have used proper casing in their comments
        importGenerator.addImport(givenName, true, path_1.default.join(externalTypesFolder, fileNominator(givenName, importedType)), false);
    }, overriddenTypes);
    var importLines = importGenerator.generateLines();
    lines.push.apply(lines, importLines);
    if (importLines.length) {
        lines.push('');
    }
    var properties = compositeType.attributes.map(function (a) {
        /** @type {string} */
        // @ts-ignore
        var rawType = a.tags.type || typeMap[a.type];
        if (!rawType) {
            console.warn("Unrecognized type: '" + a.type + "'");
            if (tableOrViewTypes.indexOf(a.type) !== -1) {
                rawType = nominators.modelNominator(a.type);
            }
            else {
                rawType = nominators.typeNominator(a.type);
            }
        }
        var typeName = (a.nullable ? rawType + " | null" : rawType);
        var modelAttributes = {
            commentLines: a.comment ? [a.comment] : [],
            optional: false,
        };
        return {
            name: nominators.propertyNominator(a.name, compositeType),
            optional: false,
            typeName: typeName,
            modelAttributes: modelAttributes,
        };
    });
    var givenName = nominators.typeNominator(compositeType.name);
    var interfaceLines = (0, generateInterface_1.default)({
        name: givenName,
        properties: properties.map(function (_a) {
            var modelAttributes = _a.modelAttributes, props = __rest(_a, ["modelAttributes"]);
            return (__assign(__assign({}, props), modelAttributes));
        }),
        comment: comment,
        exportAsDefault: true,
    });
    lines.push.apply(lines, interfaceLines);
    return lines;
};
exports.default = generateCompositeTypeFile;
