"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
Object.defineProperty(exports, "__esModule", { value: true });
var ramda_1 = require("ramda");
/**
 * @param {import('extract-pg-schema').TableOrView[]} models
 * @param {string[]} userTypes
 * @param {import('./generateModelFile').Nominators} nominators
 * @returns {string[]}
 */
function generateIndexFile(models, userTypes, nominators) {
    var isFixed = function (m) { return m.type !== 'table' || m.tags['fixed']; };
    var hasIdentifier = function (m) {
        return (0, ramda_1.filter)(function (c) { return c.isPrimary; }, m.columns).length === 1;
    };
    var creatableModels = (0, ramda_1.reject)(isFixed, models);
    var modelsWithIdColumn = (0, ramda_1.filter)(hasIdentifier, models);
    var importLine = function (m) {
        var importInitializer = !isFixed(m);
        var importId = hasIdentifier(m);
        var additionalImports = importInitializer || importId;
        var givenName = nominators.modelNominator(m.name);
        var fileName = nominators.fileNominator(givenName, m.name);
        if (!additionalImports) {
            return "import " + givenName + " from './" + fileName + "';";
        }
        else {
            var imports = __spreadArray(__spreadArray([], (importInitializer
                ? [nominators.initializerNominator(givenName, m.name)]
                : []), true), (importId ? [nominators.idNominator(givenName, m.name)] : []), true);
            return "import " + givenName + ", { " + imports.join(', ') + " } from './" + fileName + "';";
        }
    };
    var exportLine = function (m) {
        var givenName = nominators.modelNominator(m.name);
        var exportInitializer = !isFixed(m);
        var exportId = hasIdentifier(m);
        var exports = __spreadArray(__spreadArray([
            givenName
        ], (exportInitializer
            ? [nominators.initializerNominator(givenName, m.name)]
            : []), true), (exportId ? [nominators.idNominator(givenName, m.name)] : []), true);
        return "  " + exports.join(', ') + ",";
    };
    var lines = __spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray([], (0, ramda_1.map)(importLine, models), true), (0, ramda_1.map)(function (t) {
        return "import " + nominators.typeNominator(t) + " from './" + nominators.fileNominator(nominators.typeNominator(t), t) + "';";
    }, userTypes), true), [
        '',
        'type Model ='
    ], false), (0, ramda_1.map)(function (model) { return "  | " + nominators.modelNominator(model.name); }, models), true), [
        '',
        'interface ModelTypeMap {'
    ], false), (0, ramda_1.map)(function (model) { return "  '" + model.name + "': " + nominators.modelNominator(model.name) + ";"; }, models), true), [
        '}',
        '',
        'type ModelId ='
    ], false), (0, ramda_1.map)(function (model) {
        return "  | " + nominators.idNominator(nominators.modelNominator(model.name), model.name);
    }, modelsWithIdColumn), true), [
        '',
        'interface ModelIdTypeMap {'
    ], false), (0, ramda_1.map)(function (model) {
        return "  '" + model.name + "': " + nominators.idNominator(nominators.modelNominator(model.name), model.name) + ";";
    }, modelsWithIdColumn), true), [
        '}',
        ''
    ], false), (creatableModels.length > 0
        ? __spreadArray([
            'type Initializer ='
        ], (0, ramda_1.map)(function (model) {
            return "  | " + nominators.initializerNominator(nominators.modelNominator(model.name), model.name);
        }, creatableModels), true) : ['type Initializer = never']), true), [
        '',
        'interface InitializerTypeMap {'
    ], false), (0, ramda_1.map)(function (model) {
        return "  '" + model.name + "': " + nominators.initializerNominator(nominators.modelNominator(model.name), model.name) + ";";
    }, creatableModels), true), [
        '}',
        '',
        'export type {'
    ], false), (0, ramda_1.map)(exportLine, models), true), (0, ramda_1.map)(function (t) { return "  " + nominators.typeNominator(t) + ","; }, userTypes), true), [
        '',
        '  Model,',
        '  ModelTypeMap,',
        '  ModelId,',
        '  ModelIdTypeMap,',
        '  Initializer,',
        '  InitializerTypeMap',
        '};',
    ], false);
    return lines;
}
exports.default = generateIndexFile;
