"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = __importDefault(require("path"));
var ramda_1 = require("ramda");
var generateInterface_1 = __importDefault(require("./generateInterface"));
var importGenerator_1 = __importDefault(require("./importGenerator"));
/**
 * @typedef { import('extract-pg-schema').TableOrView } TableOrView
 * @typedef { import('extract-pg-schema').Type } Type
 * @typedef { import('./Model').TableModel } TableModel
 * @typedef { import('./Model').ViewModel } ViewModel
 * @typedef { import('./Config').Nominators } Nominators
 * @typedef { import('./Config').TypeMap } TypeMap
 */
/**
 * @param {TableModel | ViewModel} model
 * @param {{ typeMap: TypeMap, userTypes: (string | any)[], schemaName: string, nominators: Nominators, externalTypesFolder?: string, schemaFolderMap: {[schemaName: string]: string}, makeIdType: (innerType: string, modelName: string) => string }} p1
 * @returns {string[]}
 */
var generateModelFile = function (model, _a) {
    var typeMap = _a.typeMap, userTypes = _a.userTypes, schemaName = _a.schemaName, nominators = _a.nominators, externalTypesFolder = _a.externalTypesFolder, schemaFolderMap = _a.schemaFolderMap, makeIdType = _a.makeIdType;
    var fileNominator = nominators.fileNominator;
    var makeIdName = function (name) {
        return nominators.idNominator(nominators.modelNominator(name), name);
    };
    var lines = [];
    var comment = model.comment, tags = model.tags;
    var importGenerator = new importGenerator_1.default(schemaFolderMap[schemaName]);
    var referencedIdTypes = (0, ramda_1.pipe)(
    // @ts-ignore
    (0, ramda_1.filter)(function (p) { return Boolean(p.reference); }), (0, ramda_1.map)(function (p) { return p.reference; }), (0, ramda_1.reject)(function (p) { return p.schema === schemaName && p.table === model.name; }), 
    // @ts-ignore
    ramda_1.uniq
    // @ts-ignore
    )(model.columns);
    referencedIdTypes.forEach(function (i) {
        var givenName = nominators.modelNominator(i.table);
        importGenerator.addImport(makeIdName(i.table), false, path_1.default.join(schemaFolderMap[i.schema], fileNominator(givenName, i.table)), false);
    });
    var cols = (0, ramda_1.map)(function (_a) {
        var isArray = _a.isArray, type = _a.type, subType = _a.subType, rest = __rest(_a, ["isArray", "type", "subType"]);
        return (__assign({ type: isArray ? subType : type }, rest));
    }, model.columns);
    var appliedUserTypes = (0, ramda_1.uniq)((0, ramda_1.map)(function (p) { return p.type; }, (0, ramda_1.filter)(function (p) { return userTypes.indexOf(p.type) !== -1; }, cols)));
    appliedUserTypes.forEach(function (t) {
        var givenName = nominators.typeNominator(t);
        importGenerator.addImport(givenName, true, path_1.default.join(schemaFolderMap[schemaName], fileNominator(givenName, t)), false);
    });
    var overriddenTypes = (0, ramda_1.map)(function (p) { return p.tags.type; }, (0, ramda_1.filter)(function (p) { return Boolean(p.tags.type); }, model.columns));
    (0, ramda_1.forEach)(function (importedType) {
        var givenName = importedType; // We expect people to have used proper casing in their comments
        importGenerator.addImport(givenName, true, path_1.default.join(externalTypesFolder, fileNominator(givenName, importedType)), false);
    }, overriddenTypes);
    var primaryColumns = (0, ramda_1.filter)(function (c) { return c.isPrimary; }, model.columns);
    // If there's one and only one primary key, that's the identifier.
    var hasIdentifier = primaryColumns.length === 1;
    var properties = model.columns.map(function (c) {
        var isIdentifier = hasIdentifier && c.isPrimary;
        var idType = isIdentifier && makeIdName(model.name);
        var referenceType = c.reference && makeIdName(c.reference.table);
        var rawType = c.tags.type || idType || referenceType || typeMap[c.type];
        if (typeof rawType === 'boolean') {
            throw new Error('@type tag must include the actual type: "@type:string"');
        }
        if (typeof rawType === 'object') {
            importGenerator.addImport(rawType.name, rawType.defaultImport, rawType.absoluteImport
                ? rawType.module
                : path_1.default.join(externalTypesFolder || schemaFolderMap[schemaName], rawType.module), rawType.absoluteImport);
            rawType = rawType.name;
        }
        if (!rawType) {
            console.warn("Unrecognized type: '" + c.type + "'");
            rawType = nominators.typeNominator(c.type);
        }
        var typeName = c.nullable ? rawType + " | null" : rawType;
        var modelAttributes = {
            commentLines: c.comment ? [c.comment] : [],
            optional: false,
        };
        var initializerAttributes = {
            omit: c.generated === 'ALWAYS',
            commentLines: c.comment ? [c.comment] : [],
            optional: c.defaultValue || c.nullable,
        };
        if (c.defaultValue) {
            initializerAttributes.commentLines.push("Default value: " + c.defaultValue);
        }
        c.indices.forEach(function (index) {
            var commentLine = index.isPrimary
                ? "Primary key. Index: " + index.name
                : "Index: " + index.name;
            modelAttributes.commentLines.push(commentLine);
            initializerAttributes.commentLines.push(commentLine);
        });
        return {
            name: nominators.propertyNominator(c.name, model),
            optional: false,
            typeName: typeName,
            modelAttributes: modelAttributes,
            initializerAttributes: initializerAttributes,
        };
    });
    var importLines = importGenerator.generateLines();
    lines.push.apply(lines, importLines);
    if (importLines.length) {
        lines.push('');
    }
    if (hasIdentifier) {
        var _b = primaryColumns[0], type = _b.type, tags_1 = _b.tags;
        /** @type {string} */
        // @ts-ignore
        var innerType = tags_1.type || typeMap[type] || nominators.typeNominator(type);
        lines.push("export type " + makeIdName(model.name) + " = " + makeIdType(innerType, model.name));
        lines.push('');
    }
    var givenName = nominators.modelNominator(model.name);
    var interfaceLines = (0, generateInterface_1.default)({
        name: givenName,
        properties: properties.map(function (_a) {
            var modelAttributes = _a.modelAttributes, props = __rest(_a, ["modelAttributes"]);
            return (__assign(__assign({}, props), modelAttributes));
        }),
        comment: comment,
        exportAsDefault: true,
    });
    lines.push.apply(lines, interfaceLines);
    var generateInitializer = !tags['fixed'] && model.type === 'table';
    if (generateInitializer) {
        lines.push('');
        var initializerInterfaceLines = (0, generateInterface_1.default)({
            name: nominators.initializerNominator(givenName, model.name),
            properties: properties
                .filter(function (p) { return !p.initializerAttributes.omit; })
                .map(function (_a) {
                var initializerAttributes = _a.initializerAttributes, props = __rest(_a, ["initializerAttributes"]);
                return (__assign(__assign({}, props), initializerAttributes));
            }),
            comment: comment,
            exportAsDefault: false,
        });
        lines.push.apply(lines, initializerInterfaceLines);
    }
    return lines;
};
exports.default = generateModelFile;
