"use strict";
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var path_1 = __importDefault(require("path"));
var ImportGenerator = /** @class */ (function () {
    function ImportGenerator(srcPath) {
        this.importMap = {};
        this.srcPath = srcPath;
    }
    ImportGenerator.prototype.addImport = function (name, isDefault, absolutePath, isAbsolute) {
        var importPath = absolutePath;
        if (!isAbsolute) {
            var relativePath = path_1.default.relative(this.srcPath, absolutePath);
            // We never want Windows-style paths in our source. Fix it if necessary.
            if (path_1.default.sep === '\\') {
                relativePath = relativePath.replace(/\\/g, '/');
            }
            if (relativePath[0] !== '.') {
                relativePath = "./" + relativePath;
            }
            importPath = relativePath;
        }
        if (!(importPath in this.importMap)) {
            this.importMap[importPath] = { named: new Set() };
        }
        var importSet = this.importMap[importPath];
        if (isDefault) {
            if (importSet.default && importSet.default !== name) {
                throw new Error("Multiple default imports attempted: " + importSet.default + " and " + name + " from '" + importPath + "'");
            }
            importSet.default = name;
        }
        else {
            importSet.named.add(name);
        }
    };
    ImportGenerator.prototype.generateLines = function () {
        var _this = this;
        return Object.keys(this.importMap).map(function (relativePath) {
            return _this.generateLine(relativePath, _this.importMap[relativePath]);
        });
    };
    ImportGenerator.prototype.generateLine = function (relativePath, importSet) {
        var defaultImport = importSet.default;
        var namedImports = Array.from(importSet.named.values());
        var allImports = __spreadArray(__spreadArray([], (defaultImport ? [defaultImport] : []), true), (namedImports.length > 0 ? ["{ " + namedImports.join(', ') + " }"] : []), true).join(', ');
        return "import " + allImports + " from '" + relativePath + "';";
    };
    return ImportGenerator;
}());
exports.default = ImportGenerator;
